<?php
class pCloud {
    public $url;
    public $path;
    public $username;
    public $password;
    public $token;
    public $sslverify;

    function __construct() {
        $this->url = 'https://api.pcloud.com/';
        $this->path = get_option('pcloud_path');
        $this->username = get_option('pcloud_username');
        $this->password = get_option('pcloud_password');
        if($this->password) $this->password = pcloud_decrypt($this->password);
        $this->token = $this->token();
        $this->sslverify = isset($_SERVER['HTTPS']) && 'on' == $_SERVER['HTTPS'] ? true : false;
    }

    //获取摘要
    private function getdigest() {
        $url = $this->url . 'getdigest';
        $response = wp_remote_get($url);
        if(is_wp_error($response)) {
            return;
        }
        $data = json_decode(wp_remote_retrieve_body($response));
        return isset($data->digest) ? $data->digest : '';
    }

    //获取Token
    private function token() {
        $digest = $this->getdigest();
        if(!$digest || !$this->username || !$this->password) {
            return;
        }

        $url = $this->url . 'userinfo';
        $passworddigest = sha1($this->password . sha1($this->username) . $digest);
        $response = wp_remote_post($url, array(
            'sslverify' => $this->sslverify,
            'body' => [
                'timeformat' => 'timestamp',
                'authexpire' => 3600,
                'getauth' => 1,
                'digest' => $digest,
                'username' => $this->username,
                'passworddigest' => $passworddigest,
            ]
        ));

        if(is_wp_error($response)) {
            return;
        }
        $data = json_decode(wp_remote_retrieve_body($response));

        return isset($data->auth) ? $data->auth : '';
    }

    //退出登录
    public function logout() {
        if(!$this->token) return;

        $url = $this->url . 'logout';
        $response = wp_remote_post($url, array(
            'sslverify' => $this->sslverify,
            'body' => [
                'auth' => $this->token,
            ]
        ));

        if(is_wp_error($response)) {
            return $response->get_error_message();
        }

        return wp_remote_retrieve_body($response);
    }

    //创建文件夹
    public function createfolder($path) {
        if(!$this->token) return;

        $url = $this->url . 'createfolderifnotexists';
        $dirs = explode('/', trim($path, '/'));
        $path = $this->path;
        $folderid = 0;
        foreach($dirs as $dir) {
            $path .= '/' . $dir;
            $response = wp_remote_post($url, array(
                'sslverify' => $this->sslverify,
                'body' => [
                    'auth' => $this->token,
                    'path' => $path,
                ]
            ));

            if(is_wp_error($response)) {
                break;
            }

            $data = json_decode(wp_remote_retrieve_body($response));
            if($data->result === 0) {
                $folderid = $data->metadata->folderid;
            } else {
                $folderid = 0;
                break;
            }
        }
   
        return $folderid;
    }

    //上传
    public function upload($file, $path) {
        if(!$this->token) return '获取Token失败';
        
        require ABSPATH . WPINC . '/version.php';
        $url = $this->url . 'uploadfile';
        $filename = basename($file);

        $folderid = $this->createfolder($path);
        if(!$folderid) return '路径不正确';

        $file = new \CURLFile($file);
        $http = new WP_Http_Curl;
        $response = $http->request($url, array(
            'method' => 'POST',
            'sslverify' => $this->sslverify,
            'sslcertificates' => ABSPATH . WPINC . '/certificates/ca-bundle.crt',
            'stream' => false,
            'decompress' => false,
            'user-agent' => 'WordPress/' . $wp_version . '; ' . home_url( '/' ),
            'filename' => $filename,
            'body' => [
                'auth' => $this->token,
                'folderid' => $folderid,
                'filename' => $file
            ]
        ));

        if(is_wp_error($response)) {
            return $response->get_error_message();
        }

        $data = json_decode(wp_remote_retrieve_body($response));
        if($data->result !== 0) {
            return $data->error;
        }

        return true;
    }

    //下载
    public function download($file) {
        $url = $this->url . 'getfilelink';
        $file = trim($file, '/');
        $response = wp_remote_post($url, array(
            'sslverify' => $this->sslverify,
            'body' => [
                'auth' => $this->token,
                'path' => $this->path . '/' . $file
            ]
        ));

        if(is_wp_error($response)) {
            return;
        }

        $data = json_decode(wp_remote_retrieve_body($response));
        if($data->result === 0) {
            return 'https://' . $data->hosts[0] . $data->path;
        }

        return;
    }

    //删除
    public function delete($file) {
        $url = $this->url . 'deletefile';
        $file = trim($file, '/');
        $response = wp_remote_post($url, array(
            'sslverify' => $this->sslverify,
            'body' => [
                'auth' => $this->token,
                'path' => $this->path . '/' . $file
            ]
        ));

        if(is_wp_error($response)) {
            return $response->get_error_message();
        }

        $data = json_decode(wp_remote_retrieve_body($response));
        if($data->result !== 0) {
            return $data->error;
        }

        return true;
    }
}
?>